﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;



namespace SpaceAlert_MissionCreator
{
    public partial class RandomMissionDialog : Form
    {
        //Member Variables
        private MainFrame mainFrame;
        public List<int> forbiddenSecondsToDeploy;
        public List<Threat> threats = new List<Threat>();
        public List<int> secondsToDeployDataTransfer = new List<int>();
        public List<int> secondsToDeployIncommingData = new List<int>();
        public List<int> secondsToDeployCommunicationBlackoutShort = new List<int>();
        public List<int> secondsToDeployCommunicationBlackoutLong = new List<int>();



        public RandomMissionDialog(MainFrame mainFrame)
        {
            InitializeComponent();

            comboBox_template.SelectedIndex = 2;
            this.mainFrame = mainFrame;
        }



        /// <summary>
        /// Set Mission Parameters by Template.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboBox_template_SelectedIndexChanged(object sender, EventArgs e)
        {
            //Set Parameters by Difficulty
            switch (comboBox_template.SelectedItem.ToString())
            {
                case "Sehr einfach":
                    numericUpDown_threatCount.Value = 6;
                    numericUpDown_data.Value = 6;
                    numericUpDown_communicationBlackout.Value = 2;
                    checkBox_noInternalThreats.Checked = false;
                    checkBox_noSeriousThreats.Checked = false;
                    break;
                case "Einfach":
                    numericUpDown_threatCount.Value = 7;
                    numericUpDown_data.Value = 6;
                    numericUpDown_communicationBlackout.Value = 2;
                    checkBox_noInternalThreats.Checked = false;
                    checkBox_noSeriousThreats.Checked = false;
                    break;
                case "Normal":
                    numericUpDown_threatCount.Value = 8;
                    numericUpDown_data.Value = 5;
                    numericUpDown_communicationBlackout.Value = 3;
                    checkBox_noInternalThreats.Checked = false;
                    checkBox_noSeriousThreats.Checked = false;
                    break;
                case "Schwer":
                    numericUpDown_threatCount.Value = 9;
                    numericUpDown_data.Value = 3;
                    numericUpDown_communicationBlackout.Value = 4;
                    checkBox_noInternalThreats.Checked = false;
                    checkBox_noSeriousThreats.Checked = false;
                    break;
                case "Unmöglich":
                    numericUpDown_threatCount.Value = 10;
                    numericUpDown_data.Value = 2;
                    numericUpDown_communicationBlackout.Value = 5;
                    checkBox_noInternalThreats.Checked = false;
                    checkBox_noSeriousThreats.Checked = false;
                    break;
            }
        }



        /// <summary>
        /// Generate a new Random Mission and close the Dialog afterwards.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_generate_Click(object sender, EventArgs e)
        {
            mainFrame.init();

            //Calculate already used SecondsToDeploy
            List<int> forbiddenSecondsToDeploy = new List<int>();

            for (int i = 0; i < mainFrame.dataGridView_missionData.Rows.Count; i++)
            {
                forbiddenSecondsToDeploy.Add((int)TimeSpan.ParseExact(mainFrame.dataGridView_missionData[4, i].Value.ToString(), @"mm\:ss", CultureInfo.InvariantCulture).TotalSeconds);
            }

            //Variables
            int threatCount = (int)numericUpDown_threatCount.Value;
            int dataCount = (int)numericUpDown_data.Value;
            int dataCountDataTransfer = 0;
            int dataCountIncommingData = 0;
            int communicationBlackoutCount = (int)numericUpDown_communicationBlackout.Value;
            int communicationBlackoutCountShort = 0;
            int communicationBlackoutCountLong = 0;

            List<int> availableSecondsToDeploy = new List<int>();
            Random random = new Random();

            //Calculacte available SecondstoDeploy
            for(int i = 0; i <= 10 * 60; i += 10)
            {
                availableSecondsToDeploy.Add(i);
            }

            for(int i = 0; i < forbiddenSecondsToDeploy.Count; i++)
            {
                if (availableSecondsToDeploy.Contains(forbiddenSecondsToDeploy[i]))
                {
                    availableSecondsToDeploy.Remove(forbiddenSecondsToDeploy[i]);
                }
            }

            #region Set Threats
            while (threatCount > 0)
            {
                //Variables
                bool externalThreat = random.Next(0, 100) < 20 ? false : true; //~20% Chance for an internal Threat
                bool seriousThreat = false;
                int threatCountExternal = 0;
                int threadCountInternal = 0;

                threats.Add(new Threat());

                //Count already existing Threats
                for (int i = 0; i < threats.Count; i++)
                {
                    switch (threats[i].externalThreat)
                    {
                        case true:
                            threatCountExternal += 1;
                            break;
                        case false:
                            threadCountInternal += 1;
                            break;
                    }
                }

                //Too many external Threats existing
                if(threatCountExternal > 8)
                {
                    externalThreat = false;
                }

                //Too many internal Threats existing
                if (threadCountInternal > 8)
                {
                    externalThreat = true;
                }

                //No internal Threat Option selected
                if (checkBox_noInternalThreats.Checked)
                {
                    externalThreat = true;
                }

                //Set Threat Type
                switch (externalThreat)
                {
                    case true:
                        seriousThreat = random.Next(0, 100) < 20 ? true : false; //~20% Chance for serious Threat

                        //Not enough Points available to generate a serious Threat
                        if(threatCount < 2)
                        {
                            seriousThreat = false;
                        }

                        //No serious Threat Option selected
                        if (checkBox_noSeriousThreats.Checked)
                        {
                            seriousThreat = false;
                        }

                        threats[threats.Count - 1].externalThreat = externalThreat;
                        threats[threats.Count - 1].seriousThreat = seriousThreat;

                        break;

                    case false:
                        seriousThreat = random.Next(0, 100) < 20 ? true : false; //~20% Chance for serious internal Threat

                        //Not enough Points available to generate a serious Threat
                        if (threatCount < 2)
                        {
                            seriousThreat = false;
                        }

                        //No serious Threat Option selected
                        if (checkBox_noSeriousThreats.Checked)
                        {
                            seriousThreat = false;
                        }

                        threats[threats.Count - 1].externalThreat = externalThreat;
                        threats[threats.Count - 1].seriousThreat = seriousThreat;

                        break;
                }

                //Set TimeTPlus
                while (true)
                {
                    int timeTPlus = random.Next(1, 9);
                    bool alreadyInUse = false;

                    //Check, if the selected Time is already in Use
                    for(int i = 0; i < threats.Count - 1; i++)
                    {
                        if(threats[i].externalThreat == threats[threats.Count - 1].externalThreat)
                        {
                            if(threats[i].timeTPlus == timeTPlus)
                            {
                                alreadyInUse = true;
                                break;
                            }
                        }
                    }

                    if (!alreadyInUse)
                    {
                        threats[threats.Count - 1].timeTPlus = timeTPlus;
                        break;
                    }
                }

                threatCount -= threats[threats.Count - 1].seriousThreat ? 2 : 1;
            }

            //Sort Threats by TimeTPlus
            threats = threats.OrderBy(o => o.timeTPlus).ToList();

            //Set SecondsToDeploy for all Threats
            for (int i = 0; i < threats.Count; i++)
            {
                int interpolatedSecondsToDeploy = linear(0, 7 * 6 + 3, threats.Count - 1, i) * 10;
                int tmpValue = interpolatedSecondsToDeploy + random.Next(-3, 4) * 10;

                if(tmpValue < 0)
                {
                    i--;
                    continue;
                }

                if(tmpValue > (7 * 6 + 3) * 10)
                {
                    i--;
                    continue;
                }

                if(!availableSecondsToDeploy.Contains(tmpValue))
                {
                    i--;
                    continue;
                }

                threats[i].secondsToDeploy = tmpValue;
                availableSecondsToDeploy.Remove(tmpValue);
            }

            //Set Zone
            for (int i = 0; i < threats.Count; i++)
            {
                if (threats[i].externalThreat)
                {
                    switch(random.Next(0, 3))
                    {
                        case 0:
                            threats[i].zone = "Rot";
                            break;
                        case 1:
                            threats[i].zone = "Weiß";
                            break;
                        case 2:
                            threats[i].zone = "Blau";
                            break;
                    }
                }
            }
            #endregion

            #region setDataTransfers

            //Calculate Amount of DataTransfers
            dataCountDataTransfer = dataCount / 2;
            dataCountIncommingData = dataCount / 2;

            if(dataCount % 2 > 0)
            {
                switch(random.Next(0, 2))
                {
                    case 0:
                        dataCountDataTransfer++;
                        break;
                    case 1:
                        dataCountIncommingData++;
                        break;
                }
            }

            //Calculate SecondstoDeploy
            for(int i = 0; i < dataCount; i++)
            {
                int interpolatedSecondsToDeploy = linear(6, 10 * 6, dataCount - 1, i) * 10;
                int tmpValue = interpolatedSecondsToDeploy + random.Next(-4, 5) * 10;

                if (tmpValue < 0)
                {
                    i--;
                    continue;
                }

                if (tmpValue > (10 * 6) * 10)
                {
                    i--;
                    continue;
                }

                if (!availableSecondsToDeploy.Contains(tmpValue))
                {
                    i--;
                    continue;
                }

                //Add DataTransfer to List of Events
                switch(random.Next(0, 2))
                {
                    case 0:
                        secondsToDeployDataTransfer.Add(tmpValue);
                        availableSecondsToDeploy.Remove(tmpValue);
                        break;
                    case 1:
                        secondsToDeployIncommingData.Add(tmpValue);
                        availableSecondsToDeploy.Remove(tmpValue);
                        break;
                }
            }

            #endregion

            #region setCommunicationBlackouts

            //Calculate Amount of blackouts
            communicationBlackoutCountShort = communicationBlackoutCount / 2;
            communicationBlackoutCountLong = communicationBlackoutCount / 2;

            if (communicationBlackoutCount % 2 > 0)
            {
                switch (random.Next(0, 2))
                {
                    case 0:
                        communicationBlackoutCountShort++;
                        break;
                    case 1:
                        communicationBlackoutCountLong++;
                        break;
                }
            }

            //Calculate SecondstoDeploy
            for (int i = 0; i < communicationBlackoutCount; i++)
            {
                int interpolatedSecondsToDeploy = linear(6, 8 * 6, communicationBlackoutCount - 1, i) * 10;
                int tmpValue = interpolatedSecondsToDeploy + random.Next(-12, 13) * 10;

                if (tmpValue < 0)
                {
                    i--;
                    continue;
                }

                if (tmpValue > (10 * 6) * 10)
                {
                    i--;
                    continue;
                }

                if (!availableSecondsToDeploy.Contains(tmpValue))
                {
                    i--;
                    continue;
                }

                //Add DataTransfer to List of Events
                switch (random.Next(0, 2))
                {
                    case 0:
                        secondsToDeployCommunicationBlackoutShort.Add(tmpValue);
                        availableSecondsToDeploy.Remove(tmpValue);
                        break;
                    case 1:
                        secondsToDeployCommunicationBlackoutLong.Add(tmpValue);
                        availableSecondsToDeploy.Remove(tmpValue);
                        break;
                }
            }

            #endregion

            this.Close();
        }



        /// <summary>
        /// Closes the Dialog, discarding Changes.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_cancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }



        /// <summary>
        /// Linear Interpolation between two TimeStemps.
        /// </summary>
        /// <param name="min">Minimum Border for the Interpolation.</param>
        /// <param name="max">Maximum Border for the Interpolation.</param>
        /// <param name="maxIndex">Amount of Elements to Split the TimeLine into.</param>
        /// <param name="index">Current Element.</param>
        /// <returns></returns>
        private int linear(int min, int max, int maxIndex, int index)
        {
            return (int)Math.Round((float)(index * (max - min)) / (float)(maxIndex)) + min;
        }

    }



    /// <summary>
    /// Container Class, which defines a single Threat.
    /// </summary>
    public class Threat
    {
        //Variables
        public bool externalThreat = false;
        public bool seriousThreat = false;
        public string zone = "";
        public int timeTPlus = 0;
        public int secondsToDeploy = 0;
    }
}
